import os
import pathlib
import skia
from drawbot_skia.font import makeTTFontFromSkiaTypeface
from drawbot_skia.gstate import GraphicsState


testDir = pathlib.Path(__file__).resolve().parent
fontPath = testDir / "fonts" / "MutatorSans.ttf"


def test_font():
    skTypeface = skia.Typeface.MakeFromFile(os.fspath(fontPath))
    ttf = makeTTFontFromSkiaTypeface(skTypeface)
    assert ttf["name"].getName(6, 3, 1).toUnicode() == "MutatorMathTest-LightCondensed"
    fvar = ttf["fvar"]
    assert fvar.axes[0].axisTag == "wdth"
    assert ["name", "fvar"] == list(ttf.tables.keys())


def test_font_gs():
    gs = GraphicsState()
    gs.font(fontPath)
    assert "fvar" in gs.textStyle.ttFont


mutatorsans_variations = {
    'wdth': {
        'defaultValue': 0.0,
        'maxValue': 1000.0,
        'minValue': 0.0,
        'name': 'Width',
    },
    'wght': {
        'defaultValue': 0.0,
        'maxValue': 1000.0,
        'minValue': 0.0,
        'name': 'Weight',
    },
}


mutatorsans_instances = {
    'MutatorMathTest-LightCondensed': {'wdth': 0.0, 'wght': 0.0},
    'MutatorMathTest-BoldCondensed': {'wdth': 0.0, 'wght': 1000.0},
    'MutatorMathTest-LightWide': {'wdth': 1000.0, 'wght': 0.0},
    'MutatorMathTest-BoldWide': {'wdth': 1000.0, 'wght': 1000.0},
    'MutatorMathTest-LightCondensed_Medium_Narrow_I': {'wdth': 327.0, 'wght': 500.0},
    'MutatorMathTest-LightCondensed_Medium_Wide_I': {'wdth': 327.0, 'wght': 500.0},
}


def test_listFontVariations():
    gs = GraphicsState()
    gs.font(fontPath)
    assert mutatorsans_variations == gs.listFontVariations()


def test_listNamedInstances():
    gs = GraphicsState()
    gs.font(fontPath)
    namedInstances = gs.listNamedInstances()
    assert mutatorsans_instances == namedInstances
    assert list(mutatorsans_instances) == list(namedInstances)



size(200, 200)

bez = BezierPath()
bez.beginPath()
bez.addPoint((20, 20), "move")
bez.addPoint((20, 100), "line")
bez.addPoint((100, 100), "line")
bez.addPoint((100, 180), "line")
bez.addPoint((150, 180))
bez.addPoint((180, 150))
bez.addPoint((180, 100), "curve")
bez.addPoint((180, 50), "line")
bez.addPoint((180, 20))
bez.addPoint((150, 20), "qcurve")
bez.endPath()

fill(1, 0, 0)
stroke(0)
strokeWidth(10)
drawPath(bez)

fill(None)
stroke(1)
translate(40, 15)
scale(0.7)
lineCap("round")
lineJoin("round")

drawPath(bez)



import pytest
from drawbot_skia.segmenting import textSegments, reorderedSegments


arabicText = " أحدث "
hebrewText = " מוסיקה "
latinText = " hello "


testTexts = [
    (arabicText, 1, [(6, "Arab", 1, 0)]),
    (hebrewText, 1, [(8, "Hebr", 1, 0)]),
    (latinText, 0, [(7, "Latn", 0, 0)]),
    (latinText + arabicText + latinText, 0,
        [(8, "Latn", 0, 0), (4, "Arab", 1, 8), (2, "Arab", 0, 12), (6, "Latn", 0, 14)]),
    (arabicText + latinText + arabicText, 1,
        [(7, 'Arab', 1, 0), (5, 'Latn', 2, 7), (2, 'Latn', 1, 12), (5, 'Arab', 1, 14)]),
    (latinText + arabicText + hebrewText + latinText, 0,
        [(8, 'Latn', 0, 0), (6, 'Arab', 1, 8), (6, 'Hebr', 1, 14), (2, 'Hebr', 0, 20), (6, 'Latn', 0, 22)]),
]


@pytest.mark.parametrize("text, expectedBaseLevel, expectedSegments", testTexts)
def test_textSegments(text, expectedBaseLevel, expectedSegments):
    segments, baseLevel = textSegments(text)
    assert expectedBaseLevel == baseLevel
    segments = [(len(seg), script, bidiLevel, index) for seg, script, bidiLevel, index in segments]
    assert expectedSegments == segments


def test_reorderedSegments():
    text = latinText + arabicText + hebrewText + latinText
    segments, baseLevel = textSegments(text)
    segments = [(len(seg), script, bidiLevel, index) for seg, script, bidiLevel, index in segments]
    expectedOriginal = [(8, 'Latn', 0, 0), (6, 'Arab', 1, 8), (6, 'Hebr', 1, 14), (2, 'Hebr', 0, 20), (6, 'Latn', 0, 22)]
    assert expectedOriginal == segments
    reordered = reorderedSegments(segments, baseLevel, lambda seg: seg[2])
    expectedReordered = [(8, 'Latn', 0, 0), (6, 'Hebr', 1, 14), (6, 'Arab', 1, 8), (2, 'Hebr', 0, 20), (6, 'Latn', 0, 22)]
    assert expectedReordered == reordered
